@@ -43,7 +43,11 @@ class problem:
                 self.obj_hvp = obj_hvp 
 
 
-            self.beta = self.generate_penalty_parameter( autobeta_args = autobeta_args )
+
+            if beta == "auto":
+                self.beta = self.generate_penalty_parameter( autobeta_args = autobeta_args )
+            else:
+                self.beta = beta
 
 
 
