@@ -4,41 +4,46 @@ import numpy as np
 from .basic_manifold_np import basic_manifold_np
 
 
+
 class stiefel_np(basic_manifold_np):
     def __init__(self, var_shape) -> None:
         n = var_shape[0]
         p = var_shape[1]
         self._n = n
         self._p = p
+
+        self.dim = var_shape[:-2]
         
 
         
-        self.Ip = np.eye(self._p)
+        # self.Ip = np.eye(self._p)
+        
 
 
-        super().__init__('Stiefel',(n,p), (p,p))
+        super().__init__('Stiefel',(*self.dim, n,p), (*self.dim, p,p))
+        self.Ip = np.broadcast_to(np.eye(p), self.con_shape)
 
 
 
-    def Phi(self, M):
-        return (M + M.T )/2
+    def Phi(self, M:np.ndarray) -> np.ndarray:
+        return (M + M.swapaxes(-2,-1) )/2
 
 
 
-    def A(self, X):
-        XX = X.T @ X
+    def A(self, X:np.ndarray) -> np.ndarray:
+        XX = X.swapaxes(-2,-1) @ X
         return 1.5 * X - X @ (XX /2)
 
 
     def JA(self, X, G):
-        return G @ ( self.Ip - 0.5 * self.C(X) )  - X @ self.Phi(X.T @ G)
+        return G @ ( self.Ip - 0.5 * self.C(X) )  - X @ self.Phi(X.swapaxes(-2,-1) @ G)
 
     def JA_transpose(self,X,G):
         # JA is self-adjoint
         return self.JA(X,G)
 
     def hessA(self, X, gradf, D):
-        return - D @ self.Phi( X.T @ gradf  ) - X @ self.Phi(D.T @ gradf) - gradf @ self.Phi( D.T @ X )
+        return - D @ self.Phi( X.swapaxes(-2,-1) @ gradf  ) - X @ self.Phi(D.swapaxes(-2,-1) @ gradf) - gradf @ self.Phi( D.swapaxes(-2,-1) @ X )
 
 
     def JC(self, X, Lambda):
@@ -46,28 +51,28 @@ class stiefel_np(basic_manifold_np):
 
     
     def C(self, X):
-        return X.T @ X - self.Ip 
+        return X.swapaxes(-2,-1) @ X - self.Ip 
 
     def C_quad_penalty(self, X):
         return np.sum(self.C(X) ** 2)
 
 
     def hess_feas(self, X, D):
-        return 4*X @ self.Phi( X.T @ D ) + 2*D @ self.C(X)
+        return 4*X @ self.Phi( X.swapaxes(-2,-1) @ D ) + 2*D @ self.C(X)
 
     
 
 
 
     def Feas_eval(self, X):
-        return np.linalg.norm( self.C(X) , 'fro')
+        return np.linalg.norm( self.C(X) )
 
     def Init_point(self, Xinit = None):
         if Xinit is None:
-            Xinit = np.random.randn(self._n, self._p)
+            Xinit = np.random.randn(*self.var_shape)
         
 
-        if np.linalg.norm(Xinit.T @ Xinit - self.Ip, 'fro') > 1e-6:
+        if np.linalg.norm(Xinit.swapaxes(-2,-1) @ Xinit - self.Ip  ) > 1e-6:
             Xinit, Rinit = np.linalg.qr(Xinit)
         return Xinit
 
@@ -104,7 +109,7 @@ class stiefel_np(basic_manifold_np):
             CX = self.C(X)
             AX = X - 0.5 * X@CX
             gradf = obj_grad(AX)
-            XG = self.Phi(X.T @ gradf)
+            XG = self.Phi(X.swapaxes(-2,-1) @ gradf)
 
             # local_JA_gradf = gradf @ (np.eye(self._p) - 0.5 * CX) - X @ XG 
             
@@ -127,7 +132,7 @@ class stiefel_np(basic_manifold_np):
     # 2022/01/05
     # The simplest expression for self.generate_cdf_hess()  is
     # self.JA( X, hessf( self.JA_transpose(X,D) ) ) + self.hessA(X, gradf, D) + beta * self.hess_feas(X, D)
-    # However, it repeatively computes A(X), X.T @ D and C(X), leading to inferior efficiency in practice.
+    # However, it repeatively computes A(X), X.swapaxes(-2,-1) @ D and C(X), leading to inferior efficiency in practice.
     # A better implementation is presented below.
     # However, the function self.generate_cdf_hess()  is still not well-optimized. 
     # In the nest version I will rewrite this function for a better performance.
@@ -143,15 +148,15 @@ class stiefel_np(basic_manifold_np):
     #         CX = self.C(X)
     #         AX = X - 0.5 * X@CX
     #         gradf = obj_grad(AX)
-    #         XG = self.Phi(X.T @ gradf)
-    #         XD = self.Phi(X.T @ D)
+    #         XG = self.Phi(X.swapaxes(-2,-1) @ gradf)
+    #         XD = self.Phi(X.swapaxes(-2,-1) @ D)
 
     #         local_JAT_D = D @ (np.eye(self._p) - 0.5 * CX) - X @ XD 
     #         local_objhess_JAT_D = obj_hess(AX, local_JAT_D)
-    #         local_JA_objhess_JAT_D = local_objhess_JAT_D @ (np.eye(self._p) - 0.5 * CX) -  X @ self.Phi( X.T @ local_objhess_JAT_D )
+    #         local_JA_objhess_JAT_D = local_objhess_JAT_D @ (np.eye(self._p) - 0.5 * CX) -  X @ self.Phi( X.swapaxes(-2,-1) @ local_objhess_JAT_D )
 
 
-    #         local_hessA_objgrad_D = - D @ XG - X @ self.Phi(D.T @ gradf) - gradf @ XD
+    #         local_hessA_objgrad_D = - D @ XG - X @ self.Phi(D.swapaxes(-2,-1) @ gradf) - gradf @ XD
 
     #         local_hess_feas = 4*X @ XD + 2*D @ CX
 
@@ -168,19 +173,19 @@ class stiefel_np(basic_manifold_np):
             CX = self.C(X)
             AX = X - 0.5 * X@CX
             gradf = obj_grad(AX)
-            XG = self.Phi(X.T @ gradf)
-            XD = self.Phi(X.T @ D)
+            XG = self.Phi(X.swapaxes(-2,-1) @ gradf)
+            XD = self.Phi(X.swapaxes(-2,-1) @ D)
 
             local_JAT_D = D @ (self.Ip - 0.5 * CX) - X @ XD 
             local_objhess_JAT_D = obj_hess(AX, local_JAT_D)
-            # local_JA_objhess_JAT_D = local_objhess_JAT_D @ (np.eye(self._p) - 0.5 * CX) -  X @ self.Phi( X.T @ local_objhess_JAT_D )
+            # local_JA_objhess_JAT_D = local_objhess_JAT_D @ (np.eye(self._p) - 0.5 * CX) -  X @ self.Phi( X.swapaxes(-2,-1) @ local_objhess_JAT_D )
 
-            # local_hessA_objgrad_D = - D @ XG - X @ self.Phi(D.T @ gradf) - gradf @ XD
+            # local_hessA_objgrad_D = - D @ XG - X @ self.Phi(D.swapaxes(-2,-1) @ gradf) - gradf @ XD
 
             # local_hess_feas = 4*X @ XD + 2*D @ CX
             # return local_JA_objhess_JAT_D + local_hessA_objgrad_D + beta * local_hess_feas
 
-            return local_objhess_JAT_D @ (self.Ip - 0.5 * CX) -  X @ self.Phi( X.T @ local_objhess_JAT_D + self.Phi(D.T @ gradf) - 4* beta * XD) + D @ (2*beta*CX - XG) - gradf @ XD
+            return local_objhess_JAT_D @ (self.Ip - 0.5 * CX) -  X @ self.Phi( X.swapaxes(-2,-1) @ local_objhess_JAT_D + self.Phi(D.swapaxes(-2,-1) @ gradf) - 4* beta * XD) + D @ (2*beta*CX - XG) - gradf @ XD
 
 
 
@@ -193,19 +198,19 @@ class stiefel_np(basic_manifold_np):
             CX = self.C(X)
             AX = X
             gradf = obj_grad(AX)
-            XG = self.Phi(X.T @ gradf)
-            XD = self.Phi(X.T @ D)
+            XG = self.Phi(X.swapaxes(-2,-1) @ gradf)
+            XD = self.Phi(X.swapaxes(-2,-1) @ D)
 
             local_JAT_D = D  - X @ XD 
             local_objhess_JAT_D = obj_hess(AX, local_JAT_D)
-            # local_JA_objhess_JAT_D = local_objhess_JAT_D @ (np.eye(self._p) - 0.5 * CX) -  X @ self.Phi( X.T @ local_objhess_JAT_D )
+            # local_JA_objhess_JAT_D = local_objhess_JAT_D @ (np.eye(self._p) - 0.5 * CX) -  X @ self.Phi( X.swapaxes(-2,-1) @ local_objhess_JAT_D )
 
-            # local_hessA_objgrad_D = - D @ XG - X @ self.Phi(D.T @ gradf) - gradf @ XD
+            # local_hessA_objgrad_D = - D @ XG - X @ self.Phi(D.swapaxes(-2,-1) @ gradf) - gradf @ XD
 
             # local_hess_feas = 4*X @ XD + 2*D @ CX
             # return local_JA_objhess_JAT_D + local_hessA_objgrad_D + beta * local_hess_feas
 
-            return local_objhess_JAT_D  -  X @ self.Phi( X.T @ local_objhess_JAT_D + self.Phi(D.T @ gradf) - 4* beta * XD) + D @ (2*beta*CX - XG) - gradf @ XD
+            return local_objhess_JAT_D  -  X @ self.Phi( X.swapaxes(-2,-1) @ local_objhess_JAT_D + self.Phi(D.swapaxes(-2,-1) @ gradf) - 4* beta * XD) + D @ (2*beta*CX - XG) - gradf @ XD
 
 
 
@@ -214,52 +219,3 @@ class stiefel_np(basic_manifold_np):
         # return local_hess
 
 
-
-# import numpy as np
-# import torch
-# from torch import nn
-
-# from numpy.linalg import svd
-# from torch._C import device
-
-# class stiefel_torch:
-#     def __init__(self, n, p, device = torch.device('cpu'), dtype = torch.float64) -> None:
-#         self._n = n
-#         self._p = p
-#         self.dim = n*p 
-#         self.device = device
-#         self.dtype = dtype
-
-        
-#         self.Ip = torch.eye(self._p).to(device = self.device, dtype = self.dtype)
-
-
-
-#     def Phi(self, M):
-#         return (M + M.T)/2
-
-
-#     def A(self, X):
-#         XX = X.T @ X
-#         return 1.5 * X - X @ (XX /2)
-
-
-    
-#     def C(self, X):
-#         return X.T @ X - self.Ip
-
-#     def Feas_eval(self, X):
-#         return torch.linalg.norm( self.C(X) , 'fro')
-
-#     def Init_point(self, Xinit = None):
-#         if Xinit is None:
-#             # Xinit = np.random.randn(self._n, self._p)
-#             Xinit = torch.randn(self._n, self._p).to(device = self.device, dtype = self.dtype)
-            
-#         if self.Feas_eval(Xinit) > 1e-6:
-#             Xinit, Rinit = torch.linalg.qr(Xinit)
-#         return Xinit
-
-#     def Post_process(self,X):
-#         UX, SX, VX = torch.linalg.svd(X, full_matrices = False)
-#         return UX @ VX
\ No newline at end of file
