@@ -1,11 +1,12 @@
 from .linear import Linear_cdopt, Bilinear_cdopt, LazyLinear_cdopt
 from .conv import Conv1d_cdopt, Conv2d_cdopt, Conv3d_cdopt, ConvTranspose1d_cdopt, ConvTranspose2d_cdopt, ConvTranspose3d_cdopt
 from .rnn import RNNBase_cdopt, RNN_cdopt, LSTM_cdopt, GRU_cdopt, RNNCell_cdopt, LSTMCell_cdopt, GRUCell_cdopt
-from .utils import get_quad_penalty, get_constraint_violation, get_constraint_violation_vector,  wvt_flatten2d, wvt_flatten2d_transp, wvt_identical, wvt_transp
+from .utils import get_quad_penalty, get_constraint_violation, get_constraint_violation_vector, wvt_flatten2d, wvt_flatten2d_transp, wvt_identical, wvt_transp, get_Amapped_params, get_named_params_manifolds, get_named_Amapped_params, get_params_manifolds, parameters_to_vector
 
 
 __all__ = ["Linear_cdopt", "Bilinear_cdopt", "LazyLinear_cdopt",
  "Conv1d_cdopt", "Conv2d_cdopt", "Conv3d_cdopt",  "ConvTranspose1d_cdopt", "ConvTranspose2d_cdopt", "ConvTranspose3d_cdopt",
  "RNNBase_cdopt", "RNN_cdopt", "LSTM_cdopt", "GRU_cdopt",
 "RNNCell_cdopt", "LSTMCell_cdopt", "GRUCell_cdopt",
-"get_quad_penalty", "get_constraint_violation", "get_constraint_violation_vector", "wvt_flatten2d", "wvt_flatten2d_transp", "wvt_identical", "wvt_transp"]
\ No newline at end of file
+"get_quad_penalty", "get_constraint_violation", "get_constraint_violation_vector", "wvt_flatten2d", "wvt_flatten2d_transp", "wvt_identical", "wvt_transp", 
+"get_Amapped_params", "get_named_params_manifolds", "get_named_Amapped_params", "get_params_manifolds", "parameters_to_vector"]
\ No newline at end of file
