@@ -17,7 +17,7 @@ def get_quad_penalty(local_module: torch.nn.Module):
 
 def get_constraint_violation_vector(local_module: torch.nn.Module):
     if hasattr(local_module, 'quad_penalty'):
-        quad_all = [ local_module.quad_penalty(), ]
+        quad_all = [ torch.sqrt(local_module.quad_penalty()), ]
         
     else:
         quad_all = []
@@ -48,6 +48,23 @@ def single_module_get_params_manifold(module:nn.Module):
 
     return tmp.items()
 
+def single_module_get_Amapped_params(module:nn.Module):
+    if hasattr(module, 'manifold'):
+        
+        manifold = getattr(module, 'manifold')
+        A_map = getattr(module, 'A')
+    else:
+        manifold = None
+        A_map = lambda X: X
+        
+    members = module._parameters.items()
+    tmp = OrderedDict()
+    for k,v in members:
+        # print(k)
+        tmp[k] = A_map(v)
+
+    return tmp.items()
+
 
 def _named_members(module, get_members_fn, prefix='', recurse=True):
     r"""Helper method for yielding various names + members of modules.""" 
@@ -79,6 +96,29 @@ def get_params_manifolds(module, prefix: str = '', recurse: bool = True):
         yield  params_manifold
 
 
+def get_named_Amapped_params(module:nn.Module, prefix: str = '', recurse: bool = True):
+    '''
+    This function relies on the torch.nn.module._name_members. Therefore, the order of the outputed manifolds is exactly the same as the
+    outputs of the torch.nn.module.parameters. 
+
+    If the layer are set with no manifolds, the function will return a str; otherwise, the function
+    will return the manifolds that corresponds to the weights.
+    '''
+
+
+    # gen = _named_members(module, single_module_get_manifold, prefix = prefix, recurse = recurse)
+    gen = module._named_members(single_module_get_Amapped_params, prefix = prefix, recurse = recurse)
+    for name, Amapped_params in gen:
+        yield name, Amapped_params
+
+
+
+def get_Amapped_params(module, prefix: str = '', recurse: bool = True):
+    gen = module._named_members(single_module_get_Amapped_params, prefix = prefix, recurse = recurse)
+    for name, Amapped_params in gen:
+        yield Amapped_params
+
+
 
 def set_attributes(local_module: torch.nn.Module, attr_name:str,  value):
     if hasattr(local_module, 'attr_name'):
@@ -91,7 +131,59 @@ def set_forward_type(local_module: torch.nn.Module, forward_with_A: bool):
     set_attributes(local_module, 'forward_with_A', forward_with_A)
 
 
+
+
+def _check_param_device(param: torch.Tensor, old_param_device) -> int:
+    r"""This helper function is to check if the parameters are located
+    in the same device. Currently, the conversion between model parameters
+    and single vector form is not supported for multiple allocations,
+    e.g. parameters in different GPUs, or mixture of CPU/GPU.
+
+    Args:
+        param ([Tensor]): a Tensor of a parameter of a model
+        old_param_device (int): the device where the first parameter of a
+                                model is allocated.
+
+    Returns:
+        old_param_device (int): report device for the first time
+    """
+
+    # Meet the first parameter
+    if old_param_device is None:
+        old_param_device = param.get_device() if param.is_cuda else -1
+    else:
+        warn = False
+        if param.is_cuda:  # Check if in same GPU
+            warn = (param.get_device() != old_param_device)
+        else:  # Check if in CPU
+            warn = (old_param_device != -1)
+        if warn:
+            raise TypeError('Found two parameters on different devices, '
+                            'this is currently not supported.')
+    return old_param_device
+
+
     
+def parameters_to_vector(parameters) -> torch.Tensor:
+    r"""Convert parameters to one vector
+
+    Args:
+        parameters (Iterable[Tensor]): an iterator of Tensors that are the
+            parameters of a model.
+
+    Returns:
+        The parameters represented by a single vector
+    """
+    # Flag for the device where the parameter is located
+    param_device = None
+
+    vec = []
+    for param in parameters:
+        # Ensure the parameters are located in the same device
+        param_device = _check_param_device(param, param_device)
+
+        vec.append(param.flatten())
+    return torch.cat(vec)
 
 
 
