@@ -12,12 +12,7 @@ from .modified_apply import modified_apply
 import types
 # from . import parametrize as P
 
-try:
-    
-    import torch.nn.utils.parametrize as P
-except ImportError:
-    # parametrize is only supported for PyTorch >=1.9.0. Therefore we add a copy of the parametrize.py in the utils folder. 
-    from . import parametrize as P
+from . import parametrize as P
 
 
 def set_constraint_dissolving(module: Module, attr_name:str, manifold_class:basic_manifold = euclidean_torch, weight_var_transfer = None, manifold_args = {}, penalty_param = 0, forward_with_A:bool = True):
@@ -53,24 +48,31 @@ def set_constraint_dissolving(module: Module, attr_name:str, manifold_class:basi
     C = lambda X_tensor: manifold.C( X_tensor )
 
     
-    if forward_with_A:
-        class manifold_module(nn.Module):
-            def __init__(self) -> None:
-                super().__init__()
-                self.forward_with_A = forward_with_A
+    # if forward_with_A:
+    class manifold_module(nn.Module):
+        def __init__(self) -> None:
+            super().__init__()
+            self.forward_with_A = forward_with_A
 
-            def forward(self, X):
+        def forward(self, X):
+            if self.forward_with_A:
                 return A(X)
-
-            def right_inverse(self, AX):
-                return weight_to_var(AX)
-    else:
-        class manifold_module(nn.Module):
-            def forward(self, X):
+            else:
                 return var_to_weight(X)
 
-            def right_inverse(self, AX):
-                return weight_to_var(AX)
+        def right_inverse(self, AX):
+            return weight_to_var(AX)
+    # else:
+    #     class manifold_module(nn.Module):
+    #         def __init__(self) -> None:
+    #             super().__init__()
+    #             self.forward_with_A = forward_with_A
+
+    #         def forward(self, X):
+    #             return var_to_weight(X)
+
+    #         def right_inverse(self, AX):
+    #             return weight_to_var(AX)
 
     P.register_parametrization(module, attr_name, manifold_module())
     
